@@ -5,12 +5,26 @@ use crate::push::vector::{BoolVector, FloatVector, IntVector};
 use std::collections::HashMap;
 use std::fmt;
 
+pub const BOOL_STACK_ID: i32 = 1;
+pub const BOOL_VECTOR_STACK_ID: i32 = 2;
+pub const CODE_STACK_ID: i32 = 3;
+pub const EXEC_STACK_ID: i32 = 4;
+pub const FLOAT_STACK_ID: i32 = 5;
+pub const FLOAT_VECTOR_STACK_ID: i32 = 6;
+pub const INDEX_STACK_ID: i32 = 7;
+pub const INPUT_STACK_ID: i32 = 8;
+pub const INT_STACK_ID: i32 = 9;
+pub const INT_VECTOR_STACK_ID: i32 = 10;
+pub const NAME_STACK_ID: i32 = 11;
+pub const OUTPUT_STACK_ID: i32 = 12;
+
 pub struct PushState {
     // Scalar Types
     pub bool_stack: PushStack<bool>,
-    pub code_stack: PushStack<Item>,
+    pub code_stack: PushStack<Item>, // Change this to reference for speedup
     pub exec_stack: PushStack<Item>,
     pub float_stack: PushStack<f32>,
+    pub index_stack: PushStack<usize>,
     pub int_stack: PushStack<i32>,
     pub name_stack: PushStack<String>,
 
@@ -23,7 +37,9 @@ pub struct PushState {
     pub input_stack: PushStack<BoolVector>,
     pub output_stack: PushStack<BoolVector>,
 
+    // Bindings
     pub name_bindings: HashMap<String, Item>,
+
     pub configuration: PushConfiguration,
     pub quote_name: bool,
 }
@@ -31,11 +47,12 @@ pub struct PushState {
 impl PushState {
     pub fn new() -> Self {
         Self {
-            float_stack: PushStack::new(),
-            exec_stack: PushStack::new(),
-            code_stack: PushStack::new(),
-            int_stack: PushStack::new(),
             bool_stack: PushStack::new(),
+            code_stack: PushStack::new(),
+            exec_stack: PushStack::new(),
+            float_stack: PushStack::new(),
+            index_stack: PushStack::new(),
+            int_stack: PushStack::new(),
             name_stack: PushStack::new(),
             bool_vector_stack: PushStack::new(),
             float_vector_stack: PushStack::new(),
