@@ -3,7 +3,7 @@ use crate::push::instructions::InstructionCache;
 use crate::push::item::Item;
 use crate::push::random::CodeGenerator;
 use crate::push::state::PushState;
-use crate::push::topology::Topology;
+use crate::push::state::*;
 use std::collections::HashMap;
 use std::fmt;
 
@@ -145,6 +145,10 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("BOOLVECTOR.FLUSH"),
         Instruction::new(bool_vector_flush),
     );
+    map.insert(
+        String::from("BOOLVECTOR.ID"),
+        Instruction::new(bool_vector_id),
+    );
     map.insert(
         String::from("BOOLVECTOR.ONES"),
         Instruction::new(bool_vector_ones),
@@ -218,6 +222,10 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("INTVECTOR.DUP"),
         Instruction::new(int_vector_dup),
     );
+    map.insert(
+        String::from("INTVECTOR.DO*RANGE"),
+        Instruction::new(int_vector_do_range),
+    );
     map.insert(
         String::from("INTVECTOR.EQUAL"),
         Instruction::new(int_vector_equal),
@@ -227,8 +235,12 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         Instruction::new(int_vector_flush),
     );
     map.insert(
-        String::from("INTVECTOR.NEIGHBORS"),
-        Instruction::new(int_vector_neighbors),
+        String::from("INTVECTOR.FROMINT"),
+        Instruction::new(int_vector_from_int),
+    );
+    map.insert(
+        String::from("INTVECTOR.ID"),
+        Instruction::new(int_vector_id),
     );
     map.insert(
         String::from("INTVECTOR.ONES"),
@@ -307,6 +319,10 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
         String::from("FLOATVECTOR.FLUSH"),
         Instruction::new(float_vector_flush),
     );
+    map.insert(
+        String::from("FLOATVECTOR.ID"),
+        Instruction::new(float_vector_id),
+    );
     map.insert(
         String::from("FLOATVECTOR.ONES"),
         Instruction::new(float_vector_ones),
@@ -351,6 +367,11 @@ pub fn load_vector_instructions(map: &mut HashMap<String, Instruction>) {
 
 /////////////////////////////////////// BOOLVECTOR //////////////////////////////////////////
 
+/// BOOLVECTOR.ID: Pushes the ID of the BOOLVECTOR stack to the INTEGER stack.
+pub fn bool_vector_id(push_state: &mut PushState, _instruction_set: &InstructionCache) {
+    push_state.int_stack.push(BOOL_VECTOR_STACK_ID);
+}
+
 /// BOOLVECTOR.SET: Replaces the ith element of the top BOOLVECTOR item by the top item of the
 /// BOOLEAN stack. The index i is taken from the INTEGER stack.
 pub fn bool_vector_set(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
@@ -576,6 +597,11 @@ pub fn bool_vector_zeros(push_state: &mut PushState, _instruction_cache: &Instru
 
 /////////////////////////////////////// INTVECTOR //////////////////////////////////////////
 
+/// INTVECTOR.ID: Pushes the ID of the INTVECTOR stack to the INTEGER stack.
+pub fn int_vector_id(push_state: &mut PushState, _instruction_set: &InstructionCache) {
+    push_state.int_stack.push(INT_VECTOR_STACK_ID);
+}
+
 /// INTVECTOR.BOOLINDEX: Pushes an INTVECTOR item that contains the indices of all true values
 /// of the top BOOLVECTOR item. For example, this instruction pushes INT[0,2] if the top
 /// item on the BOOLVECTOR stack is BOOL[1,0,1]. The BOOLVECTOR item is popped.
@@ -745,26 +771,15 @@ pub fn int_vector_flush(push_state: &mut PushState, _instruction_cache: &Instruc
     push_state.int_vector_stack.flush();
 }
 
-/// INTVECTOR.NEIGHBORS: Calculates the neighborhood for a given index element and length. It
-/// pushes the indices that are contained in this neighborhood to the INTVECTOR stack.
-/// The size, the number of dimensions and index (vector topology) are taken from the INTEGER
-/// stack in that order. The radius is taken from the float stack. Distances are calculated using the
-/// Eucledian metric. All values are corrected by max-min. If the size of the top element is not a power
-/// of the dimensions the smallest hypercube that includes the indices is used to represent the
-/// topology, e.g. two dimensions and size = 38 is represented by[7,7]. Neighbor indices that
-/// do no exist (e.g. 40) are ignored.
-pub fn int_vector_neighbors(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
-    if let Some(topology) = push_state.int_stack.pop_vec(3) {
-        let size = i32::max(topology[2], 0);
-        let index = i32::max(i32::min(size - 1, topology[1]), 0) as usize;
-        let dimensions = i32::max(i32::min(size, topology[0]), 0) as usize;
-        if let Some(fval) = push_state.float_stack.pop() {
-            let radius = f32::max(fval, 0.0);
-            if let Some(neighbors) =
-                Topology::find_neighbors(&(size as usize), &dimensions, &index, &radius)
-            {
-                push_state.int_vector_stack.push(neighbors);
-            }
+/// INTVECTOR.FROMINT: Create an INTVECTOR from the elements of the INTEGER stack. The top
+/// element 0 (min-max corrected) describes the number of elements. The elements 1..n of
+/// the INTEGER stack are pushed as vector to the INTVECTOR stack.
+pub fn int_vector_from_int(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
+    if let Some(vector_size) = push_state.int_stack.pop() {
+        let size = push_state.int_stack.size() as i32;
+        let corr_size = i32::max(i32::min(size - 1, vector_size), 0) as usize;
+        if let Some(ivec) = push_state.int_stack.pop_vec(corr_size) {
+            push_state.int_vector_stack.push(IntVector::new(ivec));
         }
     }
 }
@@ -799,6 +814,26 @@ pub fn int_vector_rand(push_state: &mut PushState, _instruction_cache: &Instruct
     }
 }
 
+/// INTVECTOR.DO*RANGE: An iteration instruction that executes the top item on the execution stack
+/// once for each element of the top INTVECTOR item. The element is pushed to the INTEGER stack
+/// before the instruction of the bodyelement is pushed to the INTEGER stack
+pub fn int_vector_do_range(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
+    if let Some(mut remaining_elements) = push_state.int_vector_stack.pop() {
+        if let Some(body) = push_state.exec_stack.pop() {
+            if let Some(el) = remaining_elements.values.pop() {
+                let updated_loop = Item::list(vec![
+                    body.clone(),
+                    Item::instruction("INTVECTOR.DO*RANGE".to_string()),
+                    Item::intvec(remaining_elements),
+                ]);
+                push_state.int_stack.push(el);
+                push_state.exec_stack.push(updated_loop);
+                push_state.exec_stack.push(body);
+            }
+        }
+    }
+}
+
 /// INTVECTOR.SHOVE: Inserts the second INTEGER "deep" in the stack, at the position indexed by the
 /// top INTEGER. The index position is calculated after the index is removed.
 pub fn int_vector_shove(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
@@ -865,6 +900,11 @@ pub fn int_vector_zeros(push_state: &mut PushState, _instruction_cache: &Instruc
 
 ////////////////////////////////////// FLOATVECTOR //////////////////////////////////////////
 
+/// FLOATVECTOR.ID: Pushes the ID of the FLOATVECTOR stack to the INTEGER stack.
+pub fn float_vector_id(push_state: &mut PushState, _instruction_set: &InstructionCache) {
+    push_state.int_stack.push(FLOAT_VECTOR_STACK_ID);
+}
+
 /// FLOATVECTOR.GET: Copies the element at index i of the top FLOATVECTOR item to the FLOAT stack
 /// where i is taken from the FLOAT stack limited to valid range.
 pub fn float_vector_get(push_state: &mut PushState, _instruction_cache: &InstructionCache) {
@@ -1701,39 +1741,16 @@ mod tests {
     }
 
     #[test]
-    fn int_vector_neighbors_pushes_result_for_valid_index() {
+    fn int_vector_from_int_pushes_item() {
         let mut test_state = PushState::new();
-        test_state.float_stack.push(1.5); // Radius
-        test_state.int_stack.push(2); // Dimensions
-        test_state.int_stack.push(50); // Index
-        test_state.int_stack.push(100); // Size
-        int_vector_neighbors(&mut test_state, &icache());
+        for i in 0..10 {
+            test_state.int_stack.push(i);
+        }
+        test_state.int_stack.push(3);
+        int_vector_from_int(&mut test_state, &icache());
         assert_eq!(
             test_state.int_vector_stack.pop().unwrap(),
-            IntVector::new(vec![40, 41, 50, 51, 60, 61])
-        );
-    }
-
-    #[test]
-    fn int_vector_neighbors_corrects_out_of_bounds_index() {
-        let mut test_state = PushState::new();
-        test_state.float_stack.push(1.5); // Radius
-        test_state.int_stack.push(2); // Dimensions
-        test_state.int_stack.push(105); // Index
-        test_state.int_stack.push(100); // Size
-        int_vector_neighbors(&mut test_state, &icache());
-        assert_eq!(
-            test_state.int_vector_stack.pop().unwrap(),
-            IntVector::new(vec![88, 89, 98, 99])
-        );
-        test_state.float_stack.push(1.5); // Radius
-        test_state.int_stack.push(2); // Dimensions
-        test_state.int_stack.push(-10); // Index
-        test_state.int_stack.push(100); // Size
-        int_vector_neighbors(&mut test_state, &icache());
-        assert_eq!(
-            test_state.int_vector_stack.pop().unwrap(),
-            IntVector::new(vec![0, 1, 10, 11])
+            IntVector::new(vec![7, 8, 9])
         );
     }
 
@@ -1776,6 +1793,23 @@ mod tests {
         }
     }
 
+    #[test]
+    fn int_vector_do_range_executes_for_each_element() {
+        let mut test_state = PushState::new();
+        test_state
+            .int_vector_stack
+            .push(IntVector::new(vec![1, 2, 3, 4, 5]));
+        test_state
+            .exec_stack
+            .push(Item::instruction("NOOP".to_string()));
+        int_vector_do_range(&mut test_state, &icache());
+        assert_eq!(test_state.int_vector_stack.size(), 0);
+        assert_eq!(
+            test_state.exec_stack.to_string(),
+            "1:InstructionMeta(NOOP); 2:List: 1:Literal([1,2,3,4]); 2:InstructionMeta(INTVECTOR.DO*RANGE); 3:InstructionMeta(NOOP);;"
+        );
+    }
+
     #[test]
     fn int_vector_shove_inserts_at_right_position() {
         let mut test_state = PushState::new();
